@@ -10,8 +10,6 @@
 #include "libxml.h"
 #ifdef LIBXML_HTML_ENABLED
 
-#include <string.h> /* for memset() only ! */
-
 #ifdef HAVE_CTYPE_H
 #include <ctype.h>
 #endif
@@ -182,26 +180,6 @@ htmlSetMetaEncoding(htmlDocPtr doc, const xmlChar *encoding) {
      */
     while (cur != NULL) {
 	if (cur->name != NULL) {
-/*
-	    if (xmlStrEqual(cur->name, BAD_CAST"html"))
-		break;
-	    if (xmlStrEqual(cur->name, BAD_CAST"body")) {
-		if (encoding == NULL)
-		    return(0);
-		meta = xmlNewDocNode(doc, NULL, BAD_CAST"head", NULL);
-		xmlAddPrevSibling(cur, meta);
-		cur = meta;
-		meta = xmlNewDocNode(doc, NULL, BAD_CAST"meta", NULL);
-		xmlAddChild(cur, meta);
-		xmlNewProp(meta, BAD_CAST"http-equiv", BAD_CAST"Content-Type");
-		xmlNewProp(meta, BAD_CAST"content", BAD_CAST newcontent);
-		return(0);
-	    }
-	    if (xmlStrEqual(cur->name, BAD_CAST"head"))
-		goto found_head;
-	    if (xmlStrEqual(cur->name, BAD_CAST"meta"))
-		goto found_meta;
-*/
 	    if (xmlStrcasecmp(cur->name, BAD_CAST"html") == 0)
 		break;
 	    if (xmlStrcasecmp(cur->name, BAD_CAST"head") == 0)
@@ -220,24 +198,6 @@ htmlSetMetaEncoding(htmlDocPtr doc, const xmlChar *encoding) {
      */
     while (cur != NULL) {
 	if (cur->name != NULL) {
-/*
-	    if (xmlStrEqual(cur->name, BAD_CAST"head"))
-		break;
-	    if (xmlStrEqual(cur->name, BAD_CAST"body")) {
-		if (encoding == NULL)
-		    return(0);
-		meta = xmlNewDocNode(doc, NULL, BAD_CAST"head", NULL);
-		xmlAddPrevSibling(cur, meta);
-		cur = meta;
-		meta = xmlNewDocNode(doc, NULL, BAD_CAST"meta", NULL);
-		xmlAddChild(cur, meta);
-		xmlNewProp(meta, BAD_CAST"http-equiv", BAD_CAST"Content-Type");
-		xmlNewProp(meta, BAD_CAST"content", BAD_CAST newcontent);
-		return(0);
-	    }
-	    if (xmlStrEqual(cur->name, BAD_CAST"meta"))
-		goto found_meta;
-*/
 	    if (xmlStrcasecmp(cur->name, BAD_CAST"head") == 0)
 		break;
 	    if (xmlStrcasecmp(cur->name, BAD_CAST"meta") == 0)
@@ -334,8 +294,11 @@ found_meta:
  *									*
  ************************************************************************/
 
+void htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
+	                xmlNodePtr cur, const char *encoding, int format);
+
 static void
-htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur);
+htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur, int format);
 
 /**
  * htmlDtdDump:
@@ -381,6 +344,12 @@ static void
 htmlAttrDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
     xmlChar *value;
 
+    /*
+     * TODO: The html output method should not escape a & character
+     *       occurring in an attribute value immediately followed by
+     *       a { character (see Section B.7.1 of the HTML 4.0 Recommendation).
+     */
+
     if (cur == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"htmlAttrDump : property == NULL\n");
@@ -409,21 +378,28 @@ htmlAttrDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
  * Dump a list of HTML attributes
  */
 static void
-htmlAttrListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
+htmlAttrListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur, int format) {
+    int i = 0;
+
     if (cur == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"htmlAttrListDump : property == NULL\n");
 	return;
     }
     while (cur != NULL) {
+	i++;
+	if ((format) && (i >= 5)) {
+	    i = 0;
+	    xmlBufferWriteChar(buf, "\n");
+	}
         htmlAttrDump(buf, doc, cur);
 	cur = cur->next;
     }
 }
 
+static void
+htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int format);
 
-void
-htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur);
 /**
  * htmlNodeListDump:
  * @buf:  the HTML buffer output
@@ -433,28 +409,30 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur);
  * Dump an HTML node list, recursive behaviour,children are printed too.
  */
 static void
-htmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
+htmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int format) {
     if (cur == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"htmlNodeListDump : node == NULL\n");
 	return;
     }
     while (cur != NULL) {
-        htmlNodeDump(buf, doc, cur);
+        htmlNodeDumpFormat(buf, doc, cur, format);
 	cur = cur->next;
     }
 }
 
 /**
- * htmlNodeDump:
+ * htmlNodeDumpFormat:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @cur:  the current node
+ * @format:  should formatting spaces been added
  *
  * Dump an HTML node, recursive behaviour,children are printed too.
  */
 void
-htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
+htmlNodeDumpFormat(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur,
+	           int format) {
     htmlElemDescPtr info;
 
     if (cur == NULL) {
@@ -468,13 +446,15 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     if (cur->type == XML_DTD_NODE)
 	return;
     if (cur->type == XML_HTML_DOCUMENT_NODE) {
-	htmlDocContentDump(buf, (xmlDocPtr) cur);
+	htmlDocContentDump(buf, (xmlDocPtr) cur, format);
 	return;
     }
     if (cur->type == HTML_TEXT_NODE) {
 	if (cur->content != NULL) {
-	    if ((cur->name == xmlStringText) ||
-		(cur->name != xmlStringTextNoenc)) {
+	    if (((cur->name == xmlStringText) ||
+		 (cur->name != xmlStringTextNoenc)) &&
+		((cur->parent == NULL) ||
+		 (!xmlStrEqual(cur->parent->name, BAD_CAST "script")))) {
 		xmlChar *buffer;
 
 #ifndef XML_USE_BUFFER_CONTENT
@@ -546,11 +526,12 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     xmlBufferWriteChar(buf, "<");
     xmlBufferWriteCHAR(buf, cur->name);
     if (cur->properties != NULL)
-        htmlAttrListDump(buf, doc, cur->properties);
+        htmlAttrListDump(buf, doc, cur->properties, format);
 
     if ((info != NULL) && (info->empty)) {
         xmlBufferWriteChar(buf, ">");
-	if (cur->next != NULL) {
+	if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->next != NULL)) {
 	    if ((cur->next->type != HTML_TEXT_NODE) &&
 		(cur->next->type != HTML_ENTITY_REF_NODE))
 		xmlBufferWriteChar(buf, "\n");
@@ -559,14 +540,16 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     }
     if ((cur->content == NULL) && (cur->children == NULL)) {
         if ((info != NULL) && (info->saveEndTag != 0) &&
-	    (strcmp(info->name, "html")) && (strcmp(info->name, "body"))) {
+	    (xmlStrcmp(BAD_CAST info->name, BAD_CAST "html")) &&
+	    (xmlStrcmp(BAD_CAST info->name, BAD_CAST "body"))) {
 	    xmlBufferWriteChar(buf, ">");
 	} else {
 	    xmlBufferWriteChar(buf, "></");
 	    xmlBufferWriteCHAR(buf, cur->name);
 	    xmlBufferWriteChar(buf, ">");
 	}
-	if (cur->next != NULL) {
+	if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->next != NULL)) {
 	    if ((cur->next->type != HTML_TEXT_NODE) &&
 		(cur->next->type != HTML_ENTITY_REF_NODE))
 		xmlBufferWriteChar(buf, "\n");
@@ -589,12 +572,14 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
 	}
     }
     if (cur->children != NULL) {
-        if ((cur->children->type != HTML_TEXT_NODE) &&
+        if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->children->type != HTML_TEXT_NODE) &&
 	    (cur->children->type != HTML_ENTITY_REF_NODE) &&
 	    (cur->children != cur->last))
 	    xmlBufferWriteChar(buf, "\n");
-	htmlNodeListDump(buf, doc, cur->children);
-        if ((cur->last->type != HTML_TEXT_NODE) &&
+	htmlNodeListDump(buf, doc, cur->children, format);
+        if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->last->type != HTML_TEXT_NODE) &&
 	    (cur->last->type != HTML_ENTITY_REF_NODE) &&
 	    (cur->children != cur->last))
 	    xmlBufferWriteChar(buf, "\n");
@@ -602,30 +587,92 @@ htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
     xmlBufferWriteChar(buf, "</");
     xmlBufferWriteCHAR(buf, cur->name);
     xmlBufferWriteChar(buf, ">");
-    if (cur->next != NULL) {
+    if ((format) && (info != NULL) && (!info->isinline) &&
+	(cur->next != NULL)) {
         if ((cur->next->type != HTML_TEXT_NODE) &&
 	    (cur->next->type != HTML_ENTITY_REF_NODE))
 	    xmlBufferWriteChar(buf, "\n");
     }
 }
 
+/**
+ * htmlNodeDump:
+ * @buf:  the HTML buffer output
+ * @doc:  the document
+ * @cur:  the current node
+ *
+ * Dump an HTML node, recursive behaviour,children are printed too,
+ * and formatting returns are added.
+ */
+void
+htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur) {
+    htmlNodeDumpFormat(buf, doc, cur, 1);
+}
+
+/**
+ * htmlNodeDumpFileFormat:
+ * @out:  the FILE pointer
+ * @doc:  the document
+ * @cur:  the current node
+ * @encoding: the document encoding
+ * @format:  should formatting spaces been added
+ *
+ * Dump an HTML node, recursive behaviour,children are printed too.
+ *
+ * TODO: if encoding == NULL try to save in the doc encoding
+ *
+ * returns: the number of byte written or -1 in case of failure.
+ */
+int
+htmlNodeDumpFileFormat(FILE *out, xmlDocPtr doc,
+	               xmlNodePtr cur, const char *encoding, int format) {
+    xmlOutputBufferPtr buf;
+    xmlCharEncodingHandlerPtr handler = NULL;
+    int ret;
+
+    if (encoding != NULL) {
+	xmlCharEncoding enc;
+
+	enc = xmlParseCharEncoding(encoding);
+	if (enc != XML_CHAR_ENCODING_UTF8) {
+	    handler = xmlFindCharEncodingHandler(encoding);
+	    if (handler == NULL)
+		return(-1);
+	}
+    }
+
+    /*
+     * Fallback to HTML or ASCII when the encoding is unspecified
+     */
+    if (handler == NULL)
+	handler = xmlFindCharEncodingHandler("HTML");
+    if (handler == NULL)
+	handler = xmlFindCharEncodingHandler("ascii");
+
+    /* 
+     * save the content to a temp buffer.
+     */
+    buf = xmlOutputBufferCreateFile(out, handler);
+    if (buf == NULL) return(0);
+
+    htmlNodeDumpFormatOutput(buf, doc, cur, encoding, format);
+
+    ret = xmlOutputBufferClose(buf);
+    return(ret);
+}
+
 /**
  * htmlNodeDumpFile:
  * @out:  the FILE pointer
  * @doc:  the document
  * @cur:  the current node
  *
- * Dump an HTML node, recursive behaviour,children are printed too.
+ * Dump an HTML node, recursive behaviour,children are printed too,
+ * and formatting returns are added.
  */
 void
 htmlNodeDumpFile(FILE *out, xmlDocPtr doc, xmlNodePtr cur) {
-    xmlBufferPtr buf;
-
-    buf = xmlBufferCreate();
-    if (buf == NULL) return;
-    htmlNodeDump(buf, doc, cur);
-    xmlBufferDump(out, buf);
-    xmlBufferFree(buf);
+    htmlNodeDumpFileFormat(out, doc, cur, NULL, 1);
 }
 
 /**
@@ -636,7 +683,7 @@ htmlNodeDumpFile(FILE *out, xmlDocPtr doc, xmlNodePtr cur) {
  * Dump an HTML document.
  */
 static void
-htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur) {
+htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur, int format) {
     int type;
 
     /*
@@ -652,7 +699,7 @@ htmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur) {
 
     }
     if (cur->children != NULL) {
-        htmlNodeListDump(buf, cur, cur->children);
+        htmlNodeListDump(buf, cur, cur->children, format);
     }
     xmlBufferWriteChar(buf, "\n");
     cur->type = (xmlElementType) type;
@@ -742,8 +789,11 @@ htmlDocDumpMemory(xmlDocPtr cur, xmlChar**mem, int *size) {
  *									*
  ************************************************************************/
 
+void
+htmlDocContentDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr cur,
+	                       const char *encoding, int format);
 /**
- * htmlDtdDump:
+ * htmlDtdDumpOutput:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @encoding:  the encoding string
@@ -779,7 +829,7 @@ htmlDtdDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 }
 
 /**
- * htmlAttrDump:
+ * htmlAttrDumpOutput:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @cur:  the attribute pointer
@@ -792,6 +842,12 @@ htmlAttrDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur,
 	           const char *encoding ATTRIBUTE_UNUSED) {
     xmlChar *value;
 
+    /*
+     * TODO: The html output method should not escape a & character
+     *       occurring in an attribute value immediately followed by
+     *       a { character (see Section B.7.1 of the HTML 4.0 Recommendation).
+     */
+
     if (cur == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"htmlAttrDump : property == NULL\n");
@@ -812,7 +868,7 @@ htmlAttrDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur,
 }
 
 /**
- * htmlAttrListDump:
+ * htmlAttrListDumpOutput:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @cur:  the first attribute pointer
@@ -838,39 +894,42 @@ void htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 	                xmlNodePtr cur, const char *encoding);
 
 /**
- * htmlNodeListDump:
+ * htmlNodeListDumpOutput:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @cur:  the first node
  * @encoding:  the encoding string
+ * @format:  should formatting spaces been added
  *
  * Dump an HTML node list, recursive behaviour,children are printed too.
  */
 static void
-htmlNodeListDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, const char *encoding) {
+htmlNodeListDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
+	               xmlNodePtr cur, const char *encoding, int format) {
     if (cur == NULL) {
         xmlGenericError(xmlGenericErrorContext,
 		"htmlNodeListDump : node == NULL\n");
 	return;
     }
     while (cur != NULL) {
-        htmlNodeDumpOutput(buf, doc, cur, encoding);
+        htmlNodeDumpFormatOutput(buf, doc, cur, encoding, format);
 	cur = cur->next;
     }
 }
 
 /**
- * htmlNodeDumpOutput:
+ * htmlNodeDumpFormatOutput:
  * @buf:  the HTML buffer output
  * @doc:  the document
  * @cur:  the current node
  * @encoding:  the encoding string
+ * @format:  should formatting spaces been added
  *
  * Dump an HTML node, recursive behaviour,children are printed too.
  */
 void
-htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
-	           xmlNodePtr cur, const char *encoding) {
+htmlNodeDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
+	                 xmlNodePtr cur, const char *encoding, int format) {
     htmlElemDescPtr info;
 
     if (cur == NULL) {
@@ -889,8 +948,10 @@ htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
     }
     if (cur->type == HTML_TEXT_NODE) {
 	if (cur->content != NULL) {
-	    if ((cur->name == xmlStringText) ||
-		(cur->name != xmlStringTextNoenc)) {
+	    if (((cur->name == xmlStringText) ||
+		 (cur->name != xmlStringTextNoenc)) &&
+		((cur->parent == NULL) ||
+		 (!xmlStrEqual(cur->parent->name, BAD_CAST "script")))) {
 		xmlChar *buffer;
 
 #ifndef XML_USE_BUFFER_CONTENT
@@ -969,25 +1030,31 @@ htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 
     if ((info != NULL) && (info->empty)) {
         xmlOutputBufferWriteString(buf, ">");
-	if (cur->next != NULL) {
+	if ((format) && (!info->isinline) && (cur->next != NULL)) {
 	    if ((cur->next->type != HTML_TEXT_NODE) &&
-		(cur->next->type != HTML_ENTITY_REF_NODE))
+		(cur->next->type != HTML_ENTITY_REF_NODE) &&
+		(cur->parent != NULL) &&
+		(!xmlStrEqual(cur->parent->name, BAD_CAST "pre")))
 		xmlOutputBufferWriteString(buf, "\n");
 	}
 	return;
     }
     if ((cur->content == NULL) && (cur->children == NULL)) {
         if ((info != NULL) && (info->saveEndTag != 0) &&
-	    (strcmp(info->name, "html")) && (strcmp(info->name, "body"))) {
+	    (xmlStrcmp(BAD_CAST info->name, BAD_CAST "html")) &&
+	    (xmlStrcmp(BAD_CAST info->name, BAD_CAST "body"))) {
 	    xmlOutputBufferWriteString(buf, ">");
 	} else {
 	    xmlOutputBufferWriteString(buf, "></");
 	    xmlOutputBufferWriteString(buf, (const char *)cur->name);
 	    xmlOutputBufferWriteString(buf, ">");
 	}
-	if (cur->next != NULL) {
+	if ((format) && (cur->next != NULL) &&
+            (info != NULL) && (!info->isinline)) {
 	    if ((cur->next->type != HTML_TEXT_NODE) &&
-		(cur->next->type != HTML_ENTITY_REF_NODE))
+		(cur->next->type != HTML_ENTITY_REF_NODE) &&
+		(cur->parent != NULL) &&
+		(!xmlStrEqual(cur->parent->name, BAD_CAST "pre")))
 		xmlOutputBufferWriteString(buf, "\n");
 	}
 	return;
@@ -1007,28 +1074,51 @@ htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
 #endif 
     }
     if (cur->children != NULL) {
-        if ((cur->children->type != HTML_TEXT_NODE) &&
+        if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->children->type != HTML_TEXT_NODE) &&
 	    (cur->children->type != HTML_ENTITY_REF_NODE) &&
-	    (cur->children != cur->last))
+	    (cur->children != cur->last) &&
+	    (!xmlStrEqual(cur->name, BAD_CAST "pre")))
 	    xmlOutputBufferWriteString(buf, "\n");
-	htmlNodeListDumpOutput(buf, doc, cur->children, encoding);
-        if ((cur->last->type != HTML_TEXT_NODE) &&
+	htmlNodeListDumpOutput(buf, doc, cur->children, encoding, format);
+        if ((format) && (info != NULL) && (!info->isinline) &&
+	    (cur->last->type != HTML_TEXT_NODE) &&
 	    (cur->last->type != HTML_ENTITY_REF_NODE) &&
-	    (cur->children != cur->last))
+	    (cur->children != cur->last) &&
+	    (!xmlStrEqual(cur->name, BAD_CAST "pre")))
 	    xmlOutputBufferWriteString(buf, "\n");
     }
     xmlOutputBufferWriteString(buf, "</");
     xmlOutputBufferWriteString(buf, (const char *)cur->name);
     xmlOutputBufferWriteString(buf, ">");
-    if (cur->next != NULL) {
+    if ((format) && (info != NULL) && (!info->isinline) &&
+	(cur->next != NULL)) {
         if ((cur->next->type != HTML_TEXT_NODE) &&
-	    (cur->next->type != HTML_ENTITY_REF_NODE))
+	    (cur->next->type != HTML_ENTITY_REF_NODE) &&
+	    (cur->parent != NULL) &&
+	    (!xmlStrEqual(cur->parent->name, BAD_CAST "pre")))
 	    xmlOutputBufferWriteString(buf, "\n");
     }
 }
 
 /**
- * htmlDocContentDump:
+ * htmlNodeDumpOutput:
+ * @buf:  the HTML buffer output
+ * @doc:  the document
+ * @cur:  the current node
+ * @encoding:  the encoding string
+ *
+ * Dump an HTML node, recursive behaviour,children are printed too,
+ * and formatting returns/spaces are added.
+ */
+void
+htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
+	           xmlNodePtr cur, const char *encoding) {
+    htmlNodeDumpFormatOutput(buf, doc, cur, encoding, 1);
+}
+
+/**
+ * htmlDocContentDumpFormatOutput:
  * @buf:  the HTML buffer output
  * @cur:  the document
  * @encoding:  the encoding string
@@ -1036,7 +1126,8 @@ htmlNodeDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr doc,
  * Dump an HTML document.
  */
 void
-htmlDocContentDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr cur, const char *encoding) {
+htmlDocContentDumpFormatOutput(xmlOutputBufferPtr buf, xmlDocPtr cur,
+	                       const char *encoding, int format) {
     int type;
 
     /*
@@ -1048,12 +1139,26 @@ htmlDocContentDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr cur, const char *enco
         htmlDtdDumpOutput(buf, cur, NULL);
     }
     if (cur->children != NULL) {
-        htmlNodeListDumpOutput(buf, cur, cur->children, encoding);
+        htmlNodeListDumpOutput(buf, cur, cur->children, encoding, format);
     }
     xmlOutputBufferWriteString(buf, "\n");
     cur->type = (xmlElementType) type;
 }
 
+/**
+ * htmlDocContentDumpOutput:
+ * @buf:  the HTML buffer output
+ * @cur:  the document
+ * @encoding:  the encoding string
+ *
+ * Dump an HTML document. Formating return/spaces are added.
+ */
+void
+htmlDocContentDumpOutput(xmlOutputBufferPtr buf, xmlDocPtr cur,
+	                 const char *encoding) {
+    htmlDocContentDumpFormatOutput(buf, cur, encoding, 1);
+}
+
 /************************************************************************
  *									*
  *		Saving functions front-ends				*
@@ -1177,16 +1282,19 @@ htmlSaveFile(const char *filename, xmlDocPtr cur) {
 }
 
 /**
- * htmlSaveFileEnc:
+ * htmlSaveFileFormat:
  * @filename:  the filename
  * @cur:  the document
+ * @format:  should formatting spaces been added
+ * @encoding: the document encoding
  *
  * Dump an HTML document to a file using a given encoding.
  * 
  * returns: the number of byte written or -1 in case of failure.
  */
 int
-htmlSaveFileEnc(const char *filename, xmlDocPtr cur, const char *encoding) {
+htmlSaveFileFormat(const char *filename, xmlDocPtr cur,
+	           const char *encoding, int format) {
     xmlOutputBufferPtr buf;
     xmlCharEncodingHandlerPtr handler = NULL;
     int ret;
@@ -1226,9 +1334,26 @@ htmlSaveFileEnc(const char *filename, xmlDocPtr cur, const char *encoding) {
     buf = xmlOutputBufferCreateFilename(filename, handler, 0);
     if (buf == NULL) return(0);
 
-    htmlDocContentDumpOutput(buf, cur, encoding);
+    htmlDocContentDumpFormatOutput(buf, cur, encoding, format);
 
     ret = xmlOutputBufferClose(buf);
     return(ret);
 }
+
+/**
+ * htmlSaveFileEnc:
+ * @filename:  the filename
+ * @cur:  the document
+ * @encoding: the document encoding
+ *
+ * Dump an HTML document to a file using a given encoding
+ * and formatting returns/spaces are added.
+ * 
+ * returns: the number of byte written or -1 in case of failure.
+ */
+int
+htmlSaveFileEnc(const char *filename, xmlDocPtr cur, const char *encoding) {
+    return(htmlSaveFileFormat(filename, cur, encoding, 1));
+}
+
 #endif /* LIBXML_HTML_ENABLED */
