@@ -899,8 +899,8 @@ static int xmlLsCountNode(xmlNodePtr node) {
 	    break;
 	case XML_DOCUMENT_NODE:
 	case XML_HTML_DOCUMENT_NODE:
-#ifdef LIBXML_SGML_ENABLED
-	case XML_SGML_DOCUMENT_NODE:
+#ifdef LIBXML_DOCB_ENABLED
+	case XML_DOCB_DOCUMENT_NODE:
 #endif
 	    list = ((xmlDocPtr) node)->children;
 	    break;
@@ -1723,7 +1723,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 				    "%s is an XSLT value tree\n", arg);
 			    break;
 		    }
+#ifdef LIBXML_XPATH_ENABLED
 		    xmlXPathFreeNodeSetList(list);
+#endif
 		} else {
 		    xmlGenericError(xmlGenericErrorContext,
 			    "%s: no such node\n", arg);
@@ -1787,7 +1789,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 				    "%s is an XSLT value tree\n", arg);
 			    break;
 		    }
+#ifdef LIBXML_XPATH_ENABLED
 		    xmlXPathFreeNodeSetList(list);
+#endif
 		} else {
 		    xmlGenericError(xmlGenericErrorContext,
 			    "%s: no such node\n", arg);
@@ -1855,7 +1859,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 				    "%s is an XSLT value tree\n", arg);
 			    break;
 		    }
+#ifdef LIBXML_XPATH_ENABLED
 		    xmlXPathFreeNodeSetList(list);
+#endif
 		} else {
 		    xmlGenericError(xmlGenericErrorContext,
 			    "%s: no such node\n", arg);
