@@ -13,6 +13,36 @@
 #include <libxml/xmlerror.h>
 #include <libxml/xmlmemory.h>
 
+#define XML_GET_VAR_STR(msg, str) {				\
+    int       size;						\
+    int       chars;						\
+    char      *larger;						\
+    va_list   ap;						\
+								\
+    str = (char *) xmlMalloc(150);				\
+    if (str == NULL) 						\
+	return;							\
+								\
+    size = 150;							\
+								\
+    while (1) {							\
+	va_start(ap, msg);					\
+  	chars = vsnprintf(str, size, msg, ap);			\
+	va_end(ap);						\
+	if ((chars > -1) && (chars < size))			\
+	    break;						\
+	if (chars > -1)						\
+	    size += chars + 1;					\
+	else							\
+	    size += 100;					\
+	if ((larger = (char *) xmlRealloc(str, size)) == NULL) {\
+	    xmlFree(str);					\
+	    return;						\
+	}							\
+	str = larger;						\
+    }								\
+}
+
 /************************************************************************
  * 									*
  * 			Handling of out of context errors		*
@@ -147,6 +177,7 @@ xmlParserPrintFileContext(xmlParserInputPtr input) {
     xmlGenericError(xmlGenericErrorContext,"%s\n", content);
 }
 
+#if 0
 /**
  * xmlGetVarStr:
  * @msg:  the message format
@@ -162,6 +193,7 @@ xmlGetVarStr(const char * msg, va_list args) {
     int       length;
     int       chars, left;
     char      *str, *larger;
+    va_list   ap;
 
     str = (char *) xmlMalloc(150);
     if (str == NULL)
@@ -173,7 +205,9 @@ xmlGetVarStr(const char * msg, va_list args) {
     while (1) {
 	left = size - length;
 		    /* Try to print in the allocated space. */
-  	chars = vsnprintf(str + length, left, msg, args);
+	va_start(msg, ap);
+  	chars = vsnprintf(str + length, left, msg, ap);
+	va_end(ap);
 			  /* If that worked, we're done. */
 	if ((chars > -1) && (chars < left ))
 	    break;
@@ -190,6 +224,7 @@ xmlGetVarStr(const char * msg, va_list args) {
     }
     return(str);
 }
+#endif
 
 /**
  * xmlParserError:
@@ -207,7 +242,6 @@ xmlParserError(void *ctx, const char *msg, ...)
     xmlParserInputPtr input = NULL;
     xmlParserInputPtr cur = NULL;
     char * str;
-    va_list args;
 
     if (ctxt != NULL) {
 	input = ctxt->input;
@@ -220,9 +254,7 @@ xmlParserError(void *ctx, const char *msg, ...)
     }
 
     xmlGenericError(xmlGenericErrorContext, "error: ");
-    va_start(args, msg);
-    str = xmlGetVarStr(msg, args);
-    va_end(args);
+    XML_GET_VAR_STR(msg, str);
     xmlGenericError(xmlGenericErrorContext, str);
     if (str != NULL)
 	xmlFree(str);
@@ -253,7 +285,6 @@ xmlParserWarning(void *ctx, const char *msg, ...)
     xmlParserInputPtr input = NULL;
     xmlParserInputPtr cur = NULL;
     char * str;
-    va_list args;
 
     if (ctxt != NULL) {
 	input = ctxt->input;
@@ -266,9 +297,7 @@ xmlParserWarning(void *ctx, const char *msg, ...)
     }
         
     xmlGenericError(xmlGenericErrorContext, "warning: ");
-    va_start(args, msg);
-    str = xmlGetVarStr(msg, args);
-    va_end(args);
+    XML_GET_VAR_STR(msg, str);
     xmlGenericError(xmlGenericErrorContext, str);
     if (str != NULL)
 	xmlFree(str);
@@ -304,7 +333,6 @@ xmlParserValidityError(void *ctx, const char *msg, ...)
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
     xmlParserInputPtr input = NULL;
     char * str;
-    va_list args;
 
     if (ctxt != NULL) {
 	input = ctxt->input;
@@ -315,9 +343,7 @@ xmlParserValidityError(void *ctx, const char *msg, ...)
     }
 
     xmlGenericError(xmlGenericErrorContext, "validity error: ");
-    va_start(args, msg);
-    str = xmlGetVarStr(msg, args);
-    va_end(args);
+    XML_GET_VAR_STR(msg, str);
     xmlGenericError(xmlGenericErrorContext, str);
     if (str != NULL)
 	xmlFree(str);
@@ -342,7 +368,6 @@ xmlParserValidityWarning(void *ctx, const char *msg, ...)
     xmlParserCtxtPtr ctxt = (xmlParserCtxtPtr) ctx;
     xmlParserInputPtr input = NULL;
     char * str;
-    va_list args;
 
     if (ctxt != NULL) {
 	input = ctxt->input;
@@ -353,9 +378,7 @@ xmlParserValidityWarning(void *ctx, const char *msg, ...)
     }
         
     xmlGenericError(xmlGenericErrorContext, "validity warning: ");
-    va_start(args, msg);
-    str = xmlGetVarStr(msg, args);
-    va_end(args);
+    XML_GET_VAR_STR(msg, str);
     xmlGenericError(xmlGenericErrorContext, str);
     if (str != NULL)
 	xmlFree(str);
