@@ -69,16 +69,6 @@ typedef enum {
 
 typedef unsigned char xmlChar;
 
-/*
- * Removed in 2.3.9 ... nobody should still use this
- *
-#ifndef WIN32
-#ifndef CHAR
-#define CHAR xmlChar
-#endif
-#endif
- */
-
 /**
  * BAD_CAST:
  *
@@ -677,6 +667,9 @@ xmlChar *	xmlGetProp		(xmlNodePtr node,
 					 const xmlChar *name);
 xmlAttrPtr	xmlHasProp		(xmlNodePtr node,
 					 const xmlChar *name);
+xmlAttrPtr	xmlHasNsProp		(xmlNodePtr node,
+					 const xmlChar *name,
+					 const xmlChar *nameSpace);
 xmlAttrPtr	xmlSetNsProp		(xmlNodePtr node,
 					 xmlNsPtr ns,
 					 const xmlChar *name,
