@@ -20,10 +20,10 @@ extern "C" {
 #ifndef LIBXML2_COMPILING_MSCCDEF
 extern void xmlCheckVersion(int version);
 #endif /* LIBXML2_COMPILING_MSCCDEF */
-#define LIBXML_DOTTED_VERSION "2.3.10"
-#define LIBXML_VERSION 20310
-#define LIBXML_VERSION_STRING "20310"
-#define LIBXML_TEST_VERSION xmlCheckVersion(20310);
+#define LIBXML_DOTTED_VERSION "2.3.11"
+#define LIBXML_VERSION 20311
+#define LIBXML_VERSION_STRING "20311"
+#define LIBXML_TEST_VERSION xmlCheckVersion(20311);
 
 /**
  * WITH_TRIO:
@@ -129,7 +129,7 @@ extern void xmlCheckVersion(int version);
  *
  * Whether iconv support is available
  */
-#ifndef WIN32
+#if !defined(WIN32) || defined(__CYGWIN__)
 #if 1
 #define LIBXML_ICONV_ENABLED
 #else
