@@ -10,10 +10,6 @@
 
 #include "libxml.h"
 
-#ifdef WIN32
-#define INCLUDE_WINSOCK
-#endif
-
 #include <string.h>
 
 #include <libxml/xmlmemory.h>
@@ -1109,7 +1105,7 @@ xmlURIEscape(const xmlChar *str) {
 
     if(uri->port) {
 	xmlChar port[10];
-	snprintf(segment, 10, "%d", uri->port);
+	snprintf((char *) segment, 10, "%d", uri->port);
 	xmlStrcat(ret, BAD_CAST ":");
 	xmlStrcat(ret, port);
 	xmlFree(segment);
